@@ -10,10 +10,12 @@
 """
 import io
 import json
-from urllib.parse import urlparse
 from datetime import datetime
-import sdc_helpers.utils as utils
 import boto3
+from botocore.exceptions import ClientError as botocore_clienterror
+import sdc_helpers.utils as utils
+from sdc_helpers.redis_helper import RedisHelper
+from sdc_engine_helpers.sagemaker.utils import parse_s3_url
 from sdc_engine_helpers.date_utils import DateUtils
 
 # pylint: disable=too-few-public-methods
@@ -33,7 +35,8 @@ class DatasetStateManager:
 
     """
     date_utils = DateUtils()
-    # slack_helper = SlackHelper()
+    redis_helper = None
+    query_helper = None
     # vendor
     vendor_campaign_arn_path = 'EndpointName'
     vendor_campaign_status_path = 'EndpointStatus'
@@ -65,9 +68,13 @@ class DatasetStateManager:
         'blocking': ['Updating']
     }
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.sagemaker = boto3.client('sagemaker')
         self.s3_client = boto3.client('s3')
+        redis_config = kwargs.get('redis_config', {})
+        if not isinstance(redis_config, dict):
+            raise TypeError('Only dict is supported for redis_config')
+        self.redis_helper = RedisHelper(**redis_config)
 
     @staticmethod
     def get_vendor_solution_dataset_identifier(
@@ -114,26 +121,10 @@ class DatasetStateManager:
             "found in vendor solution".format(dataset_label)
         ))
 
-    @staticmethod
-    def parse_s3_url(url: str):
-        """Returns an (s3 bucket, key name/prefix) tuple from a url with an s3
-        scheme.
-        Args:
-            url (str):
-        Returns:
-            tuple: A tuple containing:
-                str: S3 bucket name str: S3 key
-        """
-        parsed_url = urlparse(url)
-        if parsed_url.scheme != "s3":
-            raise ValueError("Expecting 's3' scheme, got: {} in {}.".format(parsed_url.scheme, url))
-        return parsed_url.netloc, parsed_url.path.lstrip("/")
-
-    def upload_lookups(self, s3_url: str):
-        """Upload dataset from s3"""
-
+    def download_s3_blob(self, s3_url: str):
+        """download blob from s3"""
         # parse s3 url -> bucket, file_path
-        bucket, file_path = self.parse_s3_url(s3_url)
+        bucket, file_path = parse_s3_url(s3_url)
         # load data from s3 as byte steam
         data_stream = io.BytesIO()
         self.s3_client.download_fileobj(
@@ -142,8 +133,61 @@ class DatasetStateManager:
             data_stream
         )
         data_stream.seek(0)
+        return data_stream
+
+    def upload_s3_blob(self, blob: io.BytesIO, s3_url: str):
+        """download blob from s3"""
+        # parse s3 url -> bucket, file_path
+        bucket, file_path = parse_s3_url(s3_url)
+        self.s3_client.upload_fileobj(
+            blob, bucket, file_path
+        )
+
+    def download_json_blob(self, s3_url: str):
+        """download json data from s3"""
+        data_stream = self.download_s3_blob(s3_url=s3_url)
         return json.loads(data_stream.read().decode("UTF-8"))
 
+    def stream_data_between_s3(self, src_url: str, dst_url: str):
+        """
+            Streams data from source s3 url to a destination s3 url
+
+            Args:
+                src_url (str): source s3 url
+                dst_url (str): destination s3 url
+        """
+        try:
+            data_stream = self.download_s3_blob(s3_url=src_url)
+            self.upload_s3_blob(
+                blob=data_stream,
+                s3_url=dst_url
+            )
+        except botocore_clienterror as exception:
+            if exception.response['Error']['Code'] == "404":
+                # The object does not exist.
+                print((
+                    "Unable to find object. Skipping the task, "
+                    "the following exception was found = {}".format(exception)
+                ))
+            else:
+                raise
+
+    def cache_lookup(self, cache_key: str, json_data: dict):
+        """cache lookup in elasticcache
+
+        Args:
+            cache_key (str): key to use when caching
+            json_data (dict): data to cache which should be simple dictionary lookup
+        """
+        if self.redis_helper.redis_hexists(hashkey=cache_key) == 1:
+            print("This cache already exists. Deleting before caching new data")
+            self.redis_helper.redis_delete(keys=[cache_key])
+
+        self.redis_helper.redis_hmset(
+            hashkey=cache_key,
+            dict_obj=json_data
+        )
+
     def get_vendor_campaign_solution_version_arn(self, vendor_campaign: dict):
         """
             Get vendor campaign solution version status
@@ -219,8 +263,9 @@ class DatasetStateManager:
     def update_state(
             self, *,
             dataset: dict,
-            campaign: dict
-        ):
+            campaign: dict,
+            cache_key: str
+    ):
         """
             This is where jobs to create datasets are run
 
@@ -272,10 +317,33 @@ class DatasetStateManager:
             identifier = new_dataset.get('identifier', None)
             # get dataset from reference
             if isinstance(identifier, str):
-                new_dataset.update({
-                    'data':self.upload_lookups(
-                        s3_url=identifier)
-                })
+                print("Build prod destination url")
+                prod_url = identifier.replace("/train/", "/prod/")
+                print("Build previous destination url")
+                previous_url = identifier.replace("/train/", "/previous/")
+                print("Version Control: Move prod -> train")
+                self.stream_data_between_s3(
+                    src_url=prod_url,
+                    dst_url=previous_url
+                )
+
+                print("caching data in elastic cache")
+                # get data
+                json_data = self.download_json_blob(s3_url=identifier)
+
+                if not isinstance(json_data, dict):
+                    raise Exception("ClientError: Json data is not a dictionary")
+                # cache to elastic cache
+                self.cache_lookup(
+                    cache_key=cache_key,
+                    json_data=json_data
+                )
+
+                print("Version Control: Move prod -> previous")
+                self.stream_data_between_s3(
+                    src_url=identifier,
+                    dst_url=prod_url
+                )
 
         # date
         new_dataset.update({
