@@ -14,6 +14,7 @@ from sqlalchemy.exc import InternalError
 from botocore.exceptions import ClientError as botocore_clienterror
 from sdc_helpers.slack_helper import SlackHelper
 from sdc_helpers import decorators as global_decorators
+from sdc_engine_helpers.sagemaker.utils import create_resource_name
 from sdc_engine_helpers.models.subscription import Subscription
 from sdc_engine_helpers.date_utils import DateUtils
 from sdc_engine_helpers import decorators as engine_decorators
@@ -220,7 +221,7 @@ class MaintenanceManager:
     )
     def perform_dataset_maintenance(
             self, *,
-            client: dict,
+            client: object,
             engine_slug: str,
             maintenance_mode: str
     ) -> dict:
@@ -277,9 +278,19 @@ class MaintenanceManager:
                         .format(client_name=client.name)
                     )
 
+                    # build cache key
+                    cache_key = create_resource_name(
+                        context="recommendation",
+                        function=dataset['label'],
+                        additional_details=[
+                            self.service.id, client.id, engine_slug
+                        ]
+                    )
+
                     new_dataset = self.dataset_state_manager.update_state(
                         dataset=dataset,
-                        campaign=campaign
+                        campaign=campaign,
+                        cache_key=cache_key
                     )
 
                     # update dataset
@@ -361,10 +372,10 @@ class MaintenanceManager:
                     .format(client_name=client.name)
                 )
 
-                new_solution_base_name = self.query_helper.create_resource_name(
-                    service=self.service.slug,
-                    group='training',
-                    prefix=[client.name, engine_slug]
+                new_solution_base_name = create_resource_name(
+                    context=self.service.slug,
+                    function=client.name, # to mainten order
+                    additional_details=[engine_slug, 'training']
                 )
 
                 # train a new model if should_create_version True
